package T2t::Table;

use strict;
use warnings;

use Carp;
use T2t::Cell;
use T2t::Row;
use T2t::UserPreferencesCache;
use T2t::CellVisitor;

use overload q{""} => \&toString;

our @ISA = qw( T2t::HTML_Element );

sub new
{
    my $package  = shift;
    my $self     = {};

    $self->{delim}       = T2t::UserPreferencesCache::getInstance()->getDelimiter();
    $self->{caption}     = "";
    $self->{attrs}       = T2t::UserPreferencesCache::getInstance()->getTableAttributes();
    $self->{cellVisitor} = new T2t::CellVisitor();
        
    bless( $self, $package );
    
    $self->setIndividualAttribute("summary","t2t table");		# add the required summary attribute
    
    return $self;
}

sub toString
{
   my $self   = shift;
 
   # apply the squeezing, cell widths everytime before we print
   # this gives the calling method the chance to set these attributes before
   # they set the data, or use the same characteristics for multiple sets of data.
   
   $self->_applyLedger();
   
   my $preferencesCache = T2t::UserPreferencesCache::getInstance();
   
   my $result = '';
   
   # XHTML does not allow a table within a paragraph
   # so we use div instead to achieve the same effect
   
   $result .= qq(<div style="margin-top:5px;margin-bottom:5px">\n) if( $preferencesCache->isAddSpace() );
   
   $result .= $self->createTag("table", $self->{attrs}) . "\n";
   
   if( defined($self->{caption}) && $self->{caption} ne "" ) 
   {
       $result .= $self->getTab() . "<caption>$self->{caption}</caption>\n\n";
   }
   
   if( $preferencesCache->isEqualColumns() || $preferencesCache->isLimitSet() )
   {
   	   my $maxColumnCount = -1;
   	   
   	   if( $preferencesCache->isLimitSet() )
   	   {
   	   	   $maxColumnCount = $preferencesCache->getLimit();
   	   }
   	   else
   	   {
		   foreach my $row ( @{$self->{_data}} ) 
		   {
				my $columnCount = $row->getColumnCount();
				if( $columnCount > $maxColumnCount )
				{
					$maxColumnCount = $columnCount;
				}
		   }
   	   }
	   
	   foreach my $row ( @{$self->{_data}} ) 
	   {
	   		$row->limitColumns( $maxColumnCount );
	   }
   }

   if( scalar(@{$self->{_data}}) > 0 )
   {   
       $self->{_data}->[0]->setHeader( $preferencesCache->isHeader() );
   }
   
   foreach my $row ( @{$self->{_data}} ) 
   {
        $self->getCellVisitor()->affectRow( $row );
        
        $result .= $self->getTab();
        $result .= $row;
   }

   $result .= "</table>";
   $result .= "\n</div>" if( $preferencesCache->isAddSpace() );
   $result .= "\n";
   
   $result;
}

sub _applyLedger
{
   my $self = shift;
   my $colors = T2t::UserPreferencesCache::getInstance()->getAllLedgerColors();
   
   return if scalar(@{$colors}) == 0;
   
   # since the first row is not colored for a header
   # we have to move the startimg row to 1
   # also this will keep the color index in synch
   # the first row of data will always have the first
   # color

   my $start = T2t::UserPreferencesCache::getInstance()->isHeader() ? 1 : 0;
  
   for(my $i = $start; $i < scalar(@{$self->{_data}}); $i++)
   {
      my $row = $self->{_data}->[$i];
      $row->setBGColor( $colors->[ ($i-$start) % scalar(@{$colors}) ] );
   }
}

sub removeLedger
{
   my $self = shift;
      
   foreach my $row ( @{$self->{_data}} )
   {
      $row->setBGColor( undef );
   }
}

sub getData {  @{$_[0]->{_data}};  }

sub getNumberOfColumns
{
	my $self = shift;
	return scalar(@{$self->{_data}});
}

sub getRow
{
    my ($self,$index) = @_;
    @{$self->{_data}}[$index];
}

sub setRow
{
    my ($self,$index, $row) = @_;
    $self->{_data}->[$index] = $row;
}

sub addRow
{
    my ($self, $row) = @_;
    push(@{$self->{_data}}, $row);
}

sub getCaption() { $_[0]->{caption}; }
sub setCaption() { $_[0]->{caption} = $_[1]; }

sub getDelimiter() { $_[0]->{delim}; }
sub setDelimiter() { $_[0]->{delim} = $_[1]; }

sub setVAlign   {}
sub getVAlign   {0;}

sub setChar    {}
sub getChar    {0;}

sub setCharOff   {}
sub getCharOff    {0;}

sub setAlign    {  $_[0]->setIndividualAttribute("align",$_[1]); }
sub getAlign    {  $_[0]->getIndividualAttribute("align"); }

sub setBorder    {  $_[0]->setIndividualAttribute("border",$_[1]); }
sub getBorder    {  $_[0]->getIndividualAttribute("border"); }

sub setCellPadding    {  $_[0]->setIndividualAttribute("cellpadding",$_[1]); }
sub getCellPadding    {  $_[0]->getIndividualAttribute("cellpadding"); }

sub setCellSpacing    {  $_[0]->setIndividualAttribute("cellspacing",$_[1]); }
sub getCellSpacing    {  $_[0]->getIndividualAttribute("cellspacing"); }

sub setDataPageSize    {  $_[0]->setIndividualAttribute("datapagesize",$_[1]); }
sub getDataPageSize    {  $_[0]->getIndividualAttribute("datapagesize"); }

sub setFrame    {  $_[0]->setIndividualAttribute("frame",$_[1]); }
sub getFrame    {  $_[0]->getIndividualAttribute("frame"); }

sub setRules    {  $_[0]->setIndividualAttribute("rules",$_[1]); }
sub getRules    {  $_[0]->getIndividualAttribute("rules"); }

sub setSummary    {  $_[0]->setIndividualAttribute("summary",$_[1]); }
sub getSummary    {  $_[0]->getIndividualAttribute("summary"); }

sub setWidth    {  $_[0]->setIndividualAttribute("width",$_[1]); }
sub getWidth    {  $_[0]->getIndividualAttribute("width"); }

sub getCellVisitor { $_[0]->{cellVisitor}; }
sub setCellVisitor
{
    my $self = shift;
    my $visitor = shift;
    
    $self->{cellVisitor} = defined $visitor ? $visitor : new T2t::CellVisitor();
}

1;

__END__

=head NAME
 Table - <table> tag of HTML

=head1 SYNOPSIS
 use T2t:Table;
 
 Table is part of the T2t program collection.  It holds rows, which holds cells, 
 which holds the data.

=head1 ABSTRACT

Table 

=head1 AUTHOR INFORMATION

Copyright 2000-2008, Steven Scholnick <steve@scholnick.net>  

t2t is published under LGPL.  See license.html for details
